/**********************************************************************************************/
/* FBL_ArraySet.h																			  */
/*                                                                       					  */
/* Foundation Base Library ( FBL )		                                   					  */
/*                                                                       					  */
/* Copyright Paradigma, 1993-2002                                       					  */
/* All Rights Reserved.                                                 					  */
/**********************************************************************************************/

#ifndef __FBL_ArraySet_h 
	#define __FBL_ArraySet_h

#if VSDK_FRAMEWORK_STYLE
#	include <VSDK/FBL_Set.h>
#else
#	include <FBL_Set.h>
#endif // VSDK_FRAMEWORK_STYLE

#if defined(__MWERKS__)
	#pragma options align=power
#endif


/**********************************************************************************************/
class FBL_ArraySet;
class FBL_BitSet;


/**********************************************************************************************/
/*
	General purpose class.
	Implement Set of ulong numbers as array.	
	Can contain up to MAX_ULONG items.
	Array can change its size in both directions.
*/

class FBL_ArraySet : public FBL_Set
{ 	
		friend class FBL_ArraySetIterator;

    public://///////////////////////////////////////////////////////////////////////////////////

FBL_EXP							FBL_ArraySet( ulong inMaxCount ) { InitSelf( inMaxCount ); }
FBL_EXP							FBL_ArraySet( const FBL_ArraySet& inSet );
FBL_EXP							FBL_ArraySet( FBL_BitSet& inSet );
							
FBL_EXP virtual			  		~FBL_ArraySet( void );

		// Main methods 

		ulong					GetMaxCount( void )	const;
virtual ulong					GetCount( void ) const;

								// Returns reference on item at the specified position.
		ID_TYPE&	 			operator()( ulong inPos ) const;
							
FBL_EXP virtual FBL_SetIterator* GetIterator( void ) const;

		/* Sets operations: */
		// Note: for much more better perfomance use FBL_BitSet methods.
		// Methodes in this class are implemented just to be... 

FBL_EXP static FBL_ArraySet* 	MakeOperation( 		
									FBL_ArraySet* 		inLeftSet, 
									FBL_SetOperation	inOperation,
									FBL_ArraySet* 		inRightSet );

								// Returns true if includes inItem.
FBL_EXP	bool					Include( ID_TYPE inItem );
							
FBL_EXP	FBL_ArraySet*			Intersection 		( FBL_ArraySet* inList );	// AND
FBL_EXP	FBL_ArraySet*			Union		 		( FBL_ArraySet* inList );	// OR
FBL_EXP	FBL_ArraySet*			Difference			( FBL_ArraySet* inList );	// AND NOT
FBL_EXP	FBL_ArraySet*			SymmetricDifference	( FBL_ArraySet* inList );	// XOR
		

		/* Sorting operations: */
		
		bool					IsSorted( void ) const;
		void					SetSorted( bool inSorted );
		
								// Sort items by values.
FBL_EXP	void					Sort( void );
							
			 						 
		/* Methodes to work with items: */

								// Append to the end of array new item.
		void					Append( ID_TYPE inItem );
							
								// Append several items at once.
FBL_EXP	void 					AppendItems( ID_TYPE* pArray, ulong Count );

								// Append several items in reverse order at once.
FBL_EXP	void 					AppendItemsReverse( ID_TYPE* pArray, ulong Count );
	
								// Insert item to the specified position.
								// Next items moved back. 
FBL_EXP	void					InsertItem( ulong inPos, ID_TYPE inItem );
		
								// Find in array item and remove it.
								// Back items moves forward.
FBL_EXP	void					RemoveItem( ID_TYPE inItem );	
		
								// Remove item at specified position, 0..
								// Next items is moved forward.
FBL_EXP	void					RemoveAt( ulong inPos, ulong inCount = 1 );
		
								// Returns index (1..) of given inItem in set.
								// returns 0 if not found.
FBL_EXP	ulong	 				FindIndex( ID_TYPE inItem, ulong inFrom = 0 , ulong inTo = 0 );

							
	public://///////////////////////////////////////////////////////////////////////////////////
 
		/* advanced methods: */

		ID_TYPE*				begin( void ) const;	// to have direct acces.
		ID_TYPE*				end( void ) const;		// to have direct acces.
 
								// If array was changed directly throw pointers (for speed) 
								// then you must set right position of pFinish marker.
FBL_EXP	void 					SetItemsCount( ID_TYPE inNewCount );

								// Can as grow, so and reduse size of set.Can truncate exisitng data.
			 					// If NewItemsCount is 0 then free memory.
FBL_EXP	void					Resize( ulong NewItemsCount );

								// Check space before adding new data.
								// If there is no more space, resize array, but
								// not more than inMaxPossible.
FBL_EXP	void					Tune( ulong inMaxPossible );


	protected://////////////////////////////////////////////////////////////////////////////////

FBL_EXP	void					InitSelf( ID_TYPE inMaxCount );


    protected://////////////////////////////////////////////////////////////////////////////////

		ID_TYPE*				pStart;			// point on first item
		ID_TYPE*				pFinish;		// point AFTER last item
		ID_TYPE*				pStorageEnd;	

		bool					isSorted;
};

/**********************************************************************************************/
inline ulong FBL_ArraySet::GetMaxCount( void )	const	
{ 
	return (ulong)(pStorageEnd - pStart); 
}

/**********************************************************************************************/
inline ulong FBL_ArraySet::GetCount( void ) const		
{ 
	return (ulong)(pFinish - pStart);		
}	

/**********************************************************************************************/
inline bool FBL_ArraySet::IsSorted( void ) const 		
{ 
	return isSorted; 				
}

/**********************************************************************************************/
inline void FBL_ArraySet::SetSorted( bool inSorted ) 	
{ 
	isSorted = inSorted; 			
}
		
/**********************************************************************************************/
inline ID_TYPE* FBL_ArraySet::begin( void ) const	
{ 
	return pStart;	
}	

/**********************************************************************************************/
inline ID_TYPE* FBL_ArraySet::end( void ) const	
{ 
	return pFinish;	
}	


/**********************************************************************************************/
inline void FBL_ArraySet::Append( ID_TYPE inItem )
{
	if( pFinish >= pStorageEnd )	// there is no place, reallocate.
	{
		// first time we allocate 100 items, then we double the size of the array.
		ID_TYPE NewSize = (pStorageEnd - pStart) ? (ulong) 2 * (pStorageEnd - pStart) : (ulong)100;
		Resize( NewSize ); 
	}
	
	*pFinish++ = inItem;
	isSorted = false;
	
}


/**********************************************************************************************/
inline ID_TYPE& FBL_ArraySet::operator()( ulong inIndex ) const
{
	CHECK_FBL( inIndex >= 1 ); 			// range 1..
	CHECK_FBL( inIndex <= GetCount() ); 	

	return *(pStart + inIndex - 1L);
	
}


/**********************************************************************************************/
#if defined(__MWERKS__)
	#pragma options align=reset
#endif

#endif	// __FBL_ArraySet_h

